/*****************************************************
Chip type               : ATmega8L
Program type            : Application
AVR Core Clock frequency: 8,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256

if (PINB.7 ==1){ //   
PORTD.2=1; //   1
PORTD.3=1; //   2 (     :  =  )
PORTD.5=1; // 

OCR1AH=0;
OCR1AL  PWM 256  

ADC6 =   0 ... V+

SD Card = MIN 2Gb
1 sector = (256/(8000000/256/3)) sec
61 sector = 1 sec


*****************************************************/

#include <mega8.h>

#include <delay.h>

#include "mmc_if_rec_play.c"

#define ADC_VREF_TYPE 0x60



// 0 ... 488281 sector (250 Mb rec/play area)
#define MAXIMAL_SECTOR_NUMBER  488281

// if F=14.4M
#define F_TIMER_DIV_CONST                       4
// 27.46 sector = 1 sec
//      (in sector)
// 0.6 sec
#define MINIMAL_REC_SECTOR_LEN_FOR_PLAY         16
//    (in sector)
// 32 sec
#define MAXIMAL_REC_SECTOR_LEN                  878
//         (in 1/335 sec)
// 1 sec
//  CB 250
//  PMR/LPD 124
#define SHELLED_INTERVAL_REC_PAST_SQL_CLOSE     335
//         (  SHELLED_INTERVAL_REC_PAST_SQL_CLOSE) 
#define PLAY_SECTOR_AFTER_SQL_CLOSE             2
//    PTT   (in 1/335 sec)
// 0.4 sec
#define DELAY_BETWEEN_PTT_AND_PLAYBACK          135
//         PTT (in 1/335 sec)
// 0.3 sec
#define DELAY_PTT_ON_AFTER_ALL_PLAY             100



//     " ",  1  1      " "
#define SQL_OPEN_LOG_SIGNAL                     1
  

unsigned char TimerForQuantizator=0; // 8000000/256/3 = 10417 
unsigned char TimerForAnalysisSignal=0; // 8000000/256/3/42 = 248 

unsigned char SystemRPstatus; // 0=  ; 1=; 2= ; 3=    ; 4= ; 5= ,    ,  ; 6=; 7= ,; 8=    PTT  ,  ; 9=   SD 
unsigned char MemBuffer[612];
bit FlagMemBufferNeedSendToSDbuffer=0; 
bit FlagWaitWriteSDbufferToFlash=0;
bit FlagBufferToInCard=0;
bit FlagBufferToRecPlay=0;
bit FlagReadBufferInCard=0;
bit FlagReadBufferInCardOk=0;
bit FlagNeedReadNextData=0;
 

unsigned int CursorMemBufferRecPlay; //      

unsigned long int CursorSDsectorToWrite;
unsigned long int CursorSDsectorToRead;
unsigned int CounterLengRec=0;
unsigned int CounterLengRecShellInterval;
unsigned int CounterShellInterval;
unsigned char CounterSboiSDcard;

unsigned char CounterSQLopen=0;
unsigned char CounterSQLclose=0;
bit FlagSQLopen=0;
unsigned int CounterPTTonAndPause;


//     ADC     RAM   ,    ,      
void QuantizatorREC(){
unsigned char TempUchar0;
if (CursorMemBufferRecPlay <612){
TempUchar0=ADCL;
TempUchar0=ADCH;
MemBuffer[CursorMemBufferRecPlay]=TempUchar0;
CursorMemBufferRecPlay=CursorMemBufferRecPlay+1;
if (CursorMemBufferRecPlay ==512){FlagMemBufferNeedSendToSDbuffer=1;} //      "    RAM   SD "
}
} // void QuantizatorREC()



//      RAM    SD 
void WriteDataToFlashBuffer(){
unsigned int TempUInt0;
FlagMemBufferNeedSendToSDbuffer=0;
if (CursorSDsectorToWrite > MAXIMAL_SECTOR_NUMBER){CursorSDsectorToWrite=0;}
if (mmc_writesector(CursorSDsectorToWrite, MemBuffer)){
PORTD.2=0; //  
PORTD.3=0; //  
if (CounterSboiSDcard <255){CounterSboiSDcard=CounterSboiSDcard+1;}
return;
}
//  
#asm("cli")
if (CursorMemBufferRecPlay >512){ //    ?          "mmc_writesector(CursorSDsectorToWrite, MemBuffer)"   4,5   50-60    Fdsc=12khz
for (TempUInt0=512; TempUInt0<CursorMemBufferRecPlay; TempUInt0=TempUInt0+1){MemBuffer[(TempUInt0-512)]=MemBuffer[TempUInt0];} // ,     ,      RAM 
CursorMemBufferRecPlay=TempUInt0-511; // -512 + 1
}else{CursorMemBufferRecPlay=0;} 
//  
#asm("sei")
CounterSboiSDcard=0;
CounterLengRec=CounterLengRec+1;
CursorSDsectorToWrite=CursorSDsectorToWrite+1;
FlagWaitWriteSDbufferToFlash=1;
} // void WriteDataToFlashBuffer()











//          PWM 
void QuantizatorPLAY(){
if (FlagBufferToRecPlay ==0){OCR1AL=MemBuffer[CursorMemBufferRecPlay];}else{OCR1AL=MemBuffer[CursorMemBufferRecPlay+256];}
if (FlagNeedReadNextData ==1 && FlagReadBufferInCard ==0){
FlagNeedReadNextData=0;
if (FlagBufferToRecPlay ==0){
FlagBufferToInCard=1;
FlagReadBufferInCard=1; FlagReadBufferInCardOk=0;
}else{
FlagBufferToInCard=0;
if (CounterLengRec >0){FlagReadBufferInCard=1; FlagReadBufferInCardOk=0;}
}
}
if (CursorMemBufferRecPlay == 255){
FlagNeedReadNextData=1;
CursorMemBufferRecPlay=0;
if (FlagBufferToRecPlay ==0){FlagBufferToRecPlay=1;}else{FlagBufferToRecPlay=0;}
if (CounterLengRec <1){SystemRPstatus=7;}
}else{CursorMemBufferRecPlay=CursorMemBufferRecPlay+1;}
} // void QuantizatorPLAY()






//       
void ReadDataInFlashToBuffer(){
int TempInt0;
if (CursorSDsectorToRead > MAXIMAL_SECTOR_NUMBER){CursorSDsectorToRead=0;}
if (FlagBufferToInCard ==0){
PORTD.2=1;
TempInt0=mmc_readsector_start256(CursorSDsectorToRead, MemBuffer);
}else{
PORTD.2=0;
TempInt0=mmc_readsector_next256(MemBuffer+256);
CursorSDsectorToRead=CursorSDsectorToRead+1;
if (CounterLengRec>0){CounterLengRec=CounterLengRec-1;}
}
if (TempInt0 ==0){
FlagReadBufferInCardOk=1;
CounterSboiSDcard=0;
}else{
PORTD.2=0; //  
PORTD.3=0; //  
if (CounterSboiSDcard <255){CounterSboiSDcard=CounterSboiSDcard+1;}
}
FlagReadBufferInCard=0;
} // void ReadDataInFlashToBuffer()









//***************************************   **********************************************************//



// Timer1 overflow interrupt service routine
//  8000000/256 = 31250   
interrupt [TIM1_OVF] void timer1_ovf_isr(void){
bit TmpFlag0=0;

TimerForQuantizator=TimerForQuantizator+1;
if (TimerForQuantizator ==F_TIMER_DIV_CONST){
TimerForQuantizator=0;
if (SystemRPstatus ==1 || SystemRPstatus ==3){QuantizatorREC();}
if (SystemRPstatus ==6){QuantizatorPLAY();}
}
if (TimerForQuantizator ==1){

TimerForAnalysisSignal=TimerForAnalysisSignal+1;
if (TimerForAnalysisSignal ==42){ //  248   
TimerForAnalysisSignal=0;
if (PINB.7 == SQL_OPEN_LOG_SIGNAL){CounterSQLopen=0; if (CounterSQLclose <255){CounterSQLclose++;}}else{{CounterSQLclose=0; if (CounterSQLopen <255){CounterSQLopen++;}}}  
if (CounterSQLopen >10){FlagSQLopen=1;}
if (CounterSQLclose >5){FlagSQLopen=0;}
} // if (TimerForAnalysisSignal ==42){ //  248   

if (TimerForAnalysisSignal ==30){ //  248   
if (FlagSQLopen ==1){ //  else 
// ST 

if (SystemRPstatus ==0 && CounterSboiSDcard ==0 && !FlagMemBufferNeedSendToSDbuffer && !FlagWaitWriteSDbufferToFlash){
PORTD.5=0; //  
SystemRPstatus=1;
CursorMemBufferRecPlay=0; //      
CounterLengRec=0;
CursorSDsectorToRead=CursorSDsectorToWrite;
}

if (SystemRPstatus ==3){SystemRPstatus=1;}

if (SystemRPstatus ==1 && CounterLengRec > MAXIMAL_REC_SECTOR_LEN && !FlagMemBufferNeedSendToSDbuffer && !FlagWaitWriteSDbufferToFlash){
// ST     MAXIMAL_REC_SECTOR_LEN         
SystemRPstatus=4;
FlagBufferToInCard=0;
FlagReadBufferInCard=1;
FlagReadBufferInCardOk=0;
// EN     MAXIMAL_REC_SECTOR_LEN         
// SystemRPstatus=2; //          
}


// EN 
}else{ // if (FlagSQLopen ==1){ //  else 
// ST 

if (SystemRPstatus ==1){CounterLengRecShellInterval=CounterLengRec; CounterShellInterval=0; SystemRPstatus=3;}
if (SystemRPstatus ==2){TmpFlag0=1;}
if (SystemRPstatus ==3){
CounterShellInterval=CounterShellInterval+1;
if (CounterShellInterval > SHELLED_INTERVAL_REC_PAST_SQL_CLOSE){
CounterLengRec=CounterLengRecShellInterval+PLAY_SECTOR_AFTER_SQL_CLOSE;
TmpFlag0=1;
}
}

if (TmpFlag0 ==1){
if (CounterLengRec > MINIMAL_REC_SECTOR_LEN_FOR_PLAY && CounterSboiSDcard ==0){
SystemRPstatus=4;
FlagReadBufferInCard=1;
FlagBufferToInCard=0;
FlagReadBufferInCardOk=0;
}else{
SystemRPstatus=0;
}
}

// EN 
} // else if (FlagSQLopen ==1){ //  else 
} // if (TimerForAnalysisSignal ==30){ //  248    




if (TimerForAnalysisSignal ==35){ //  248   
if (SystemRPstatus == 4 && FlagReadBufferInCardOk ==1){
CursorMemBufferRecPlay=0;
FlagNeedReadNextData=1;
FlagBufferToRecPlay=0; //     / : 0=_0; 1=_1
SystemRPstatus=5;
CounterPTTonAndPause=0;
PORTD.5=1; //  
}

if (SystemRPstatus == 5){
CounterPTTonAndPause=CounterPTTonAndPause+1;
if (CounterPTTonAndPause > DELAY_BETWEEN_PTT_AND_PLAYBACK){
SystemRPstatus=6; //  
}
}


if (SystemRPstatus ==7){
OCR1AL=127;
CounterSQLopen=0;
CounterSQLclose=0;
if (DELAY_PTT_ON_AFTER_ALL_PLAY >0){
SystemRPstatus=8;
CounterShellInterval=0;
}else{
PORTD.5=0; //  
SystemRPstatus=0;
}
}

if (SystemRPstatus ==8){
CounterShellInterval=CounterShellInterval+1;
if (CounterShellInterval > DELAY_PTT_ON_AFTER_ALL_PLAY){
PORTD.5=0; //  
SystemRPstatus=0;
}
}


} // if (TimerForAnalysisSignal ==35){ //  248    


if (SystemRPstatus ==0){
CounterLengRec=CounterLengRec+1;
if (CounterLengRec == 2500){CounterLengRec=0; SystemRPstatus=9;}
}


} // if (TimerForQuantizator ==1)

} // interrupt [TIM1_OVF] void timer1_ovf_isr(void)










void main(void){
// Declare your local variables here
unsigned char CounterForBlinkSvetodiode2=0;
bit FlagAddCounterForBlinkSvetodiode2=0;

OSCCAL=255;

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=Out Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=0 State0=T 
PORTB=0x00;
DDRB=0x02;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=Out Func4=In Func3=Out Func2=Out Func1=In Func0=In 
// State7=T State6=T State5=0 State4=T State3=0 State2=0 State1=T State0=T 
PORTD=0x00;
DDRD=0x2C;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
TCCR0=0x00;
TCNT0=0x00;


// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 8000,000 kHz
// Mode: Fast PWM top=00FFh
// OC1A output: Non-Inv.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: On
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x81;
TCCR1B=0x09;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x04;


// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 250,000 kHz
// ADC Voltage Reference: AVCC pin
// Only the 8 most significant bits of
// the AD conversion result are used
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0b11100101; // + free running 
ADMUX=6 | (ADC_VREF_TYPE & 0xff); // 6  
ADCSRA|=0b01000000; //     

SystemRPstatus=9;

// Global enable interrupts
#asm("sei")
PORTD.2=1; //  
PORTD.3=1; //  
OCR1AL=127;
delay_ms(2000);
PORTD.3=0; //  
PORTD.5=1; //  
delay_ms(2000);
PORTD.2=0; //  
PORTD.3=0; //  
PORTD.5=0; //  
delay_ms(500);

while (mmc_init() !=0){delay_ms(100);} 
if (mmc_readsector_start256(MAXIMAL_SECTOR_NUMBER, MemBuffer) ==0){
mmc_readsector_next256(MemBuffer);
PORTD.3=1; //  
PORTD.2=1; //  
CounterSboiSDcard=0;
SystemRPstatus=0;
}


while (1){

if (FlagWaitWriteSDbufferToFlash){
if (spi_byte(0xFF)){FlagWaitWriteSDbufferToFlash=0; PORTD.2=0;} //    Flash      
}else{
if (FlagMemBufferNeedSendToSDbuffer){WriteDataToFlashBuffer(); PORTD.2=1;}else{if (FlagReadBufferInCard){ReadDataInFlashToBuffer();}}
}




if (SystemRPstatus ==0){
if (TimerForAnalysisSignal ==0 && FlagAddCounterForBlinkSvetodiode2){
FlagAddCounterForBlinkSvetodiode2=0;
CounterForBlinkSvetodiode2=CounterForBlinkSvetodiode2+1;
if (CounterForBlinkSvetodiode2 == 127){PORTD.2=1;}
if (CounterForBlinkSvetodiode2 == 255){PORTD.2=0;}
}
if (TimerForAnalysisSignal ==1){FlagAddCounterForBlinkSvetodiode2=1;}
}


if (SystemRPstatus ==9){
if (mmc_readsector_start256(MAXIMAL_SECTOR_NUMBER, MemBuffer) !=0){
CounterSboiSDcard=255;
}else{
mmc_readsector_next256(MemBuffer);
SystemRPstatus=0; CounterSboiSDcard=0;
FlagWaitWriteSDbufferToFlash=0;
FlagMemBufferNeedSendToSDbuffer=0;
}
}
if (CounterSboiSDcard > 20){
SystemRPstatus=9;
PORTD.2=0; //  
PORTD.3=0; //  
PORTD.5=0; //  
while (mmc_init() !=0){delay_ms(200);} 
PORTD.2=1; //  
FlagWaitWriteSDbufferToFlash=0;
FlagMemBufferNeedSendToSDbuffer=0;
FlagReadBufferInCard=0; //      ? 1=
} // if (CounterSboiSDcard > 20)



};
}
